@@ -245,7 +245,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
    */
   function readU32(): Decoded32 {
     const bytes = readBytes(MAX_NUMBER_OF_BYTE_U32);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeUInt32(buffer);
   }
@@ -254,7 +254,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
     // where 32 bits = max 4 bytes
 
     const bytes = readBytes(4);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeUInt32(buffer);
   }
@@ -263,7 +263,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
     // where 7 bits = max 1 bytes
 
     const bytes = readBytes(1);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeUInt32(buffer);
   }
@@ -273,7 +273,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
    */
   function read32(): Decoded32 {
     const bytes = readBytes(MAX_NUMBER_OF_BYTE_U32);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeInt32(buffer);
   }
@@ -283,14 +283,14 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
    */
   function read64(): Decoded64 {
     const bytes = readBytes(MAX_NUMBER_OF_BYTE_U64);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeInt64(buffer);
   }
 
   function readU64(): Decoded64 {
     const bytes = readBytes(MAX_NUMBER_OF_BYTE_U64);
-    const buffer = Buffer.from(bytes);
+    const buffer = new Uint8Array(bytes);
 
     return decodeUInt64(buffer);
   }
@@ -382,9 +382,14 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
       if (type == constants.types.func) {
         dump([type], "func");
 
-        const paramValtypes: Array<Valtype> = parseVec(
-          (b) => constants.valtypes[b]
-        );
+        const paramValtypes: Array<Valtype> = parseVec((b) => {
+          const valtype = constants.valtypes[b];
+          if (valtype === undefined) {
+            throw new Error(`unexpected value type ${b}`);
+          }
+
+          return valtype;
+        });
         const params = paramValtypes.map((v) => t.funcParam(/*valtype*/ v));
 
         const result: Array<Valtype> = parseVec((b) => constants.valtypes[b]);
@@ -648,9 +653,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
 
       const code: Array<Instruction> = [];
 
-      /**
-       * Parse locals
-       */
+      // Parse locals
       const funcLocalNumU32 = readU32();
       const funcLocalNum = funcLocalNumU32.value;
       eatBytes(funcLocalNumU32.nextIndex);
@@ -721,6 +724,13 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
         eatBytes(1);
       }
 
+      // Table instructions
+      // https://webassembly.github.io/spec/core/binary/instructions.html#table-instructions
+      if (instructionByte === 0xfc) {
+        instructionByte = 0xfc00 + readByte();
+        eatBytes(1);
+      }
+
       const instruction = constants.symbolsByByte[instructionByte];
 
       if (typeof instruction === "undefined") {
@@ -729,6 +739,12 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
         );
       }
 
+      if (instruction === "illegal") {
+        throw new Error(
+          `tried to decode an illegal bytecode: ${toHex(instructionByte)}`
+        );
+      }
+
       if (typeof instruction.object === "string") {
         dump([instructionByte], `${instruction.object}.${instruction.name}`);
       } else {
@@ -752,19 +768,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
       if (instruction.name === "loop") {
         const startLoc = getPosition();
 
-        const blocktypeByte = readByte();
-        eatBytes(1);
-
-        const blocktype = constants.blockTypes[blocktypeByte];
-
-        dump([blocktypeByte], "blocktype");
-
-        if (typeof blocktype === "undefined") {
-          throw new CompileError(
-            "Unexpected blocktype: " + toHex(blocktypeByte)
-          );
-        }
-
+        const blocktype = parseBlockType();
         const instr = [];
 
         parseInstructionBlock(instr);
@@ -781,19 +785,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
       } else if (instruction.name === "if") {
         const startLoc = getPosition();
 
-        const blocktypeByte = readByte();
-        eatBytes(1);
-
-        const blocktype = constants.blockTypes[blocktypeByte];
-
-        dump([blocktypeByte], "blocktype");
-
-        if (typeof blocktype === "undefined") {
-          throw new CompileError(
-            "Unexpected blocktype: " + toHex(blocktypeByte)
-          );
-        }
-
+        const blocktype = parseBlockType();
         const testIndex = t.withRaw(t.identifier(getUniqueName("if")), "");
         const ifBody = [];
         parseInstructionBlock(ifBody);
@@ -829,19 +821,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
       } else if (instruction.name === "block") {
         const startLoc = getPosition();
 
-        const blocktypeByte = readByte();
-        eatBytes(1);
-
-        const blocktype = constants.blockTypes[blocktypeByte];
-
-        dump([blocktypeByte], "blocktype");
-
-        if (typeof blocktype === "undefined") {
-          throw new CompileError(
-            "Unexpected blocktype: " + toHex(blocktypeByte)
-          );
-        }
-
+        const blocktype = parseBlockType();
         const instr = [];
         parseInstructionBlock(instr);
 
@@ -1138,7 +1118,7 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
 
     if (typeof elementType === "undefined") {
       throw new CompileError(
-        "Unknown element type in table: " + toHex(elementType)
+        "Unknown element type in table: " + toHex(elementTypeByte)
       );
     }
 
@@ -1374,44 +1354,191 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
       const startLoc = getPosition();
 
       const tableindexu32 = readU32();
-      const tableindex = tableindexu32.value;
+      const bitfield = tableindexu32.value;
       eatBytes(tableindexu32.nextIndex);
 
-      dump([tableindex], "table index");
+      dump([bitfield], "bitfield");
 
-      /**
-       * Parse instructions
-       */
-      const instr = [];
-      parseInstructionBlock(instr);
+      if (bitfield === 0) {
+        // Parse instructions
+        const instr = [];
+        parseInstructionBlock(instr);
 
-      /**
-       * Parse ( vector function index ) *
-       */
-      const indicesu32 = readU32();
-      const indices = indicesu32.value;
-      eatBytes(indicesu32.nextIndex);
+        // Parse ( vector function index ) *
+        const indicesu32 = readU32();
+        const indices = indicesu32.value;
+        eatBytes(indicesu32.nextIndex);
 
-      dump([indices], "num indices");
+        dump([indices], "num indices");
 
-      const indexValues = [];
+        const indexValues = [];
 
-      for (let i = 0; i < indices; i++) {
-        const indexu32 = readU32();
-        const index = indexu32.value;
-        eatBytes(indexu32.nextIndex);
+        for (let i = 0; i < indices; i++) {
+          const indexu32 = readU32();
+          const index = indexu32.value;
+          eatBytes(indexu32.nextIndex);
 
-        dump([index], "index");
+          dump([index], "index");
 
-        indexValues.push(t.indexLiteral(index));
+          indexValues.push(t.indexLiteral(index));
+        }
+
+        const elemNode = WITH_LOC(
+          t.elem(t.indexLiteral(bitfield), instr, indexValues),
+          startLoc
+        );
+
+        elems.push(elemNode);
+      } else if (bitfield === 1) {
+        const elemKind = readByte();
+        eatBytes(1);
+
+        if (elemKind !== 0) {
+          throw new Error(`unexpected Elem kind: ${toHex(elemKind)}`);
+        }
+
+        // Parse ( vector function index ) *
+        const indicesu32 = readU32();
+        const indices = indicesu32.value;
+        eatBytes(indicesu32.nextIndex);
+
+        dump([indices], "num indices");
+
+        const indexValues = [];
+
+        for (let i = 0; i < indices; i++) {
+          const indexu32 = readU32();
+          const index = indexu32.value;
+          eatBytes(indexu32.nextIndex);
+
+          dump([index], "index");
+
+          indexValues.push(t.indexLiteral(index));
+        }
+
+        // TODO: emit a AST node, for now just make it parse.
+      } else if (bitfield === 2) {
+        const u32 = readU32();
+        const tableidx = u32.value;
+        eatBytes(u32.nextIndex);
+
+        dump([tableidx], "tableidx");
+
+        // Parse instructions
+        const instr = [];
+        parseInstructionBlock(instr);
+
+        const elemKind = readByte();
+        eatBytes(1);
+
+        if (elemKind !== 0) {
+          throw new Error(`unexpected Elem kind: ${toHex(elemKind)}`);
+        }
+
+        // Parse ( vector function index ) *
+        const indicesu32 = readU32();
+        const indices = indicesu32.value;
+        eatBytes(indicesu32.nextIndex);
+
+        dump([indices], "num indices");
+
+        const indexValues = [];
+
+        for (let i = 0; i < indices; i++) {
+          const indexu32 = readU32();
+          const index = indexu32.value;
+          eatBytes(indexu32.nextIndex);
+
+          dump([index], "index");
+
+          indexValues.push(t.indexLiteral(index));
+        }
+
+        const elemNode = WITH_LOC(
+          t.elem(t.indexLiteral(bitfield), instr, indexValues),
+          startLoc
+        );
+
+        elems.push(elemNode);
+      } else if (bitfield === 3) {
+        const elemKind = readByte();
+        eatBytes(1);
+
+        if (elemKind !== 0) {
+          throw new Error(`unexpected Elem kind: ${toHex(elemKind)}`);
+        }
+
+        // Parse ( vector function index ) *
+        const countU32 = readU32();
+        const count = countU32.value;
+        eatBytes(countU32.nextIndex);
+
+        dump([count], "count");
+
+        for (let i = 0; i < count; i++) {
+          const indexu32 = readU32();
+          const index = indexu32.value;
+          eatBytes(indexu32.nextIndex);
+
+          dump([index], "index");
+        }
+
+        // TODO: emit a AST node, for now just make it parse.
+      } else if (bitfield === 4) {
+        const expr = [];
+        parseInstructionBlock(expr);
+
+        const countU32 = readU32();
+        const count = countU32.value;
+        eatBytes(countU32.nextIndex);
+
+        dump([count], "count");
+
+        for (let i = 0; i < count; i++) {
+          const code = [];
+          parseInstructionBlock(code);
+        }
+
+        // TODO: emit a AST node, for now just make it parse.
+      } else if (bitfield === 5) {
+        const reftype = readByte();
+        eatBytes(1);
+
+        dump([reftype], "reftype");
+
+        const countU32 = readU32();
+        const count = countU32.value;
+        eatBytes(countU32.nextIndex);
+
+        dump([count], "count");
+
+        for (let i = 0; i < count; i++) {
+          const code = [];
+          parseInstructionBlock(code);
+        }
+
+        // TODO: emit a AST node, for now just make it parse.
+      } else if (bitfield === 7) {
+        const reftype = readByte();
+        eatBytes(1);
+
+        dump([reftype], "reftype");
+
+        const countU32 = readU32();
+        const count = countU32.value;
+        eatBytes(countU32.nextIndex);
+
+        dump([count], "count");
+
+        for (let i = 0; i < count; i++) {
+          const code = [];
+          parseInstructionBlock(code);
+        }
+
+        // TODO: emit a AST node, for now just make it parse.
+      } else {
+        throw new Error(`unexpected Elem with bitfield ${toHex(bitfield)}`);
       }
-
-      const elemNode = WITH_LOC(
-        t.elem(t.indexLiteral(tableindex), instr, indexValues),
-        startLoc
-      );
-
-      elems.push(elemNode);
     }
 
     return elems;
@@ -1888,6 +2015,31 @@ export function decode(ab: ArrayBuffer, opts: DecoderOpts): Program {
     }
   }
 
+  function parseBlockType() {
+    const blocktypeByte = readByte();
+    const blocktype = constants.blockTypes[blocktypeByte];
+
+    if (typeof blocktype !== "undefined") {
+      eatBytes(1);
+
+      dump([blocktypeByte], "blocktype");
+
+      // value type
+      return blocktype;
+    } else {
+      // type index
+      const u32 = readU32();
+      eatBytes(u32.nextIndex);
+
+      const signature = state.typesInModule[u32.value];
+      console.log({ signature });
+
+      dump([u32.value], "typeidx");
+
+      return u32.value;
+    }
+  }
+
   parseModuleHeader();
   parseVersion();
 
