@@ -2,7 +2,7 @@
 
 const {
   getFixtures,
-  compareWithExpected
+  compareWithExpected,
 } = require("@webassemblyjs/helper-test-framework");
 
 const { parse } = require("../lib");
@@ -12,7 +12,7 @@ describe("compiler", () => {
   describe("wast", () => {
     describe("tokenizing", () => {
       const testSuites = getFixtures(__dirname, "tokenizer", "**/actual.wast");
-      const pre = f => JSON.stringify(tokenize(f), null, 2);
+      const pre = (f) => JSON.stringify(tokenize(f), null, 2);
 
       compareWithExpected(testSuites, pre, "expected.json");
     });
@@ -20,7 +20,7 @@ describe("compiler", () => {
     describe("parsing", () => {
       const testSuites = getFixtures(__dirname, "fixtures", "**/actual.wast");
 
-      const pre = f => {
+      const pre = (f) => {
         const ast = parse(f);
 
         return JSON.stringify(ast, null, 2);
